#pragma once
#include <list>
#include <vcg/Matrix44.h>

namespace vcg {


class CISGNode 
{
public:
	virtual	~CISGNode(void){};
	
	virtual void glDraw(const float DocTime)=0;
};

class CISGGroup :public CISGNode 
{
	public:
  virtual ~CISGGroup();
	typedef std::list<CISGNode *>::iterator iterator;
  
  
	std::list<CISGNode *> Sons;
	virtual void glDraw(const float DocTime);
};

class CISGTransformation :public CISGNode 
{
	public:
  Matrix44f m;
	virtual void glDraw(const float DocTime);
};

class CISGTranslation :public CISGNode 
{
	public:
	CISGTranslation::CISGTranslation() :V(Point3f(0,0,0)){};
	CISGTranslation::CISGTranslation(Point3f &v): V(v) {};
  Point3f V;
	virtual void glDraw(const float DocTime) { glTranslate(V); };
};

class CISGRotation :public CISGNode 
{
	public:
	CISGRotation::CISGRotation() :Axis(Point3f(0,0,1)),AngleDeg(0){};
	CISGRotation::CISGRotation(float ad, Point3f &v): Axis(v),AngleDeg(ad) {};

  Point3f Axis;
	float AngleDeg; 
  virtual void glDraw(const float DocTime){ glRotatef(AngleDeg,Axis[0],Axis[1],Axis[2]); };
};

class CISGScaling :public CISGNode 
{
	public:
		CISGScaling(const float v):axis(v,v,v){};
		CISGScaling(Point3f &v):axis(v){};

  Point3f axis;
	virtual void glDraw(const float DocTime){glScale(axis);};
};

class CISGAnimRotation :public CISGNode 
{
	public:
  Point3f axis;
	float AngularSpeedDPS; //Degree Per Sec;
  float StartAngleDeg; 
	virtual void glDraw(const float DocTime);
};

class CISGAnimZPrecession :public CISGNode 
{
	public:
  CISGAnimZPrecession();
 
  float DeclinationDeg;
	float AngularSpeedDPS; //Degree Per Sec;
  float StartAngleDeg; 
	virtual ~CISGAnimZPrecession(){};
	virtual void glDraw(const float DocTime);
};

class CISGGround :public CISGNode 
{
	public:
		CISGGround(){};
 
	virtual ~CISGGround(){};
	virtual void glDraw(const float DocTime);
};

// Minimal Scene graph
// Solo i gruppi definiscono uno scope di trasformazioni
// l'ordine dei figli  significativo.
class CISG
{
public:
	CISG(void);
	~CISG(void);
	CISGGroup root;
};


} // end namespace